#include <util.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdlib.h>
#include <Profile.h>
#include <getopt.h>
#include <string.h>
#include <vector>

#define MAX_COMPS 50


/**
 * @brief 
 * @param argc
 * @param argv
 * @return 
 */
Profile :: Profile(int argc, char **argv)
{
	int opt;

	fileName_ = "";
	title_ = "";

	nComps_ = 0;
	nSteps_ = 0;
	gridSize_ = 0;

	index_ = 0;
	ticMarks_ = 0;
	legendPos_ = 0;
	decoration_ = true;
	grid_ = false;
	verbose_ = false;
	debug_ = false;
	border_ = 0.00;
	lineWidth_ = 1.5;
	printXdim_ = 5.0;
	printYdim_ = 3.0;
	fontSize_ = 17;

	while ((opt = getopt (argc, argv, "i:p:t:c:vdh")) != EOF)
	{
		switch (opt)
		{
		case 'i':			// input file
			fileName_ = optarg;
			break;
		case 'p':			// projection
            {
            optind--;
/*
            // Use the code below to get multiple entries for an option.
            string comps("");
            for( ;optind < argc && *argv[optind] != '-'; optind++) {
                if ( comps != "" ) comps = comps + " ";
			    comps = comps + string(argv[optind]);
            }
			argvComps_.push_back(comps);
*/
			argvComps_.push_back(optarg);
			argvTypes_.push_back(PROJECTION);
			break;
                }
		case 't':			// triangular projection
			argvComps_.push_back(optarg);
			argvTypes_.push_back(TRIANGLE);
			break;
		case 'c':			// color
			extractColors(optarg);
			break;
		case 'v':			// verbose
			verbose_ = true;
			break;
		case 'd':			// debug
			debug_ = true;
			break;
		case 'h':			// help
			printUsage (EXIT_SUCCESS);
			break;
		case '?':
			if (optopt == 'i' || optopt == 'p' || optopt == 'm' || isprint (optopt))
			{
				printUsage (EXIT_FAILURE);
			}
		}
	}

	if ( argvComps_.size() == 0 )
	{
		cerr << "Please, enter at least on projection to plot." << endl;
		printUsage(EXIT_FAILURE);
	}

	if ( argvColors_.size() == 0 )
	{
		extractColors("green:blue:red:purple:yellow:pink:lightblue");
	}

	if ( readFile() )
	{
		cerr << "Wrong file format - '" << fileName_ << "'" << endl;
		printUsage(EXIT_FAILURE);
	}

	if ( verbose_ )
	{
		cout << endl;
		cout << "Input data file name: " << fileName_ << endl;
		cout << "Title: " << title_ << endl;
		cout << "Grid size: " << gridSize_ << endl;
		cout << "Number of components: " << nComps_ << endl;
		cout << "Number of projection: " << argvComps_.size() << endl;
		for ( int i = 0; i < argvComps_.size(); i++ )
		{
			cout << "    Projection " << i << " - components: " << argvComps_[i] << endl;
		}
		cout << endl;
		cout << endl;
	}
}


void Profile :: extractColors(string c)
{
	// Strip color names
	stringstream ss (c);

	while ( ss.good() )
	{
		string substr;
		getline (ss, substr, ':');
		argvColors_.push_back(substr);
	}
}


//-------------------------------------------


void Profile :: extractInfoFromComment(string line)
{
    auto titleFound = stripeInfo(line, "Title");
    auto coordinatesFound = stripeInfo(line, "Coordinates");
    auto colorsFound = stripeInfo(line, "Colors");
    auto stepTimeFound = stripeInfo(line, "Time");

    istringstream iss(line);

    if ( titleFound || coordinatesFound || colorsFound || stepTimeFound )
    {
        do
        {
            string word;
            iss >> word;
            auto i = 0;

            if ( !word.empty() ) 
            {
                if ( titleFound )
                {
                    //cout << "Title: " << word << endl;
                    title_ += word;
                }

                if ( coordinatesFound )
                {
                    //cout << "Coordinate[" << coordinates_.size() << "]: " << word << endl;
                    coordinates_.push_back(word);
                }

                if ( colorsFound )
                {
                    //cout << "Colors[" << colors_.size() << "]: " << word << endl;
                    colors_.push_back(word);
                }

                if ( stepTimeFound )
                {
                    //cout << "Time: " << word << endl;
                    stepTime_.push_back(stringToDouble(word));
                }
            }
        }
        while (iss);
    }

    nComps_ = coordinates_.size();
}


void Profile :: extractData(void)
{

    ifstream file;
    file.open (fileName_);

    string line; line.clear();    
    auto commentFound = false;
    auto emptyLineFound = false;

    // Read file line by line
    while (!file.eof())
    {
        getline(file, line);

        // Check if it is a comment line
        auto commentPos = line.find("#");
        if ( commentPos == 0 ) {
            //cout << "This is a comment. " << endl;
            commentFound = true;

            // Check if it has any information tag
            extractInfoFromComment(line);

        }
        else {
            //cout << "This is NOT a comment. " << endl;
            commentFound = false;
        }


        // Check if the line is empty (will start new group of data,e.i.a new "step") 
        if ( isEmpty(line) ) {
            if ( !emptyLineFound ) 
            {
                vector<vector <float> > step;
                data_.push_back(step);  // Create a "step" in data multi dim vector
                emptyLineFound = true;
            }
            //cout << "EMPTY LINE" << endl;
        } 
        // If line is not empty and not a comment so it contains data (coordinates)
        else if ( !commentFound ) {
            istringstream iss(line);
            emptyLineFound = false;

            vector<float> row;
            data_[data_.size()-1].push_back(row);

            // Split line into words
            do
            {
                string word;
                iss >> word;

                if ( !word.empty() )
                {
                    //cout << "Word: " << word;
                    if ( checkDouble(word) ) {
                        //cout << " : is a double" << endl;
                        data_[data_.size()-1][ data_[data_.size() -1].size()-1 ].push_back(stringToDouble(word));
                    }
                    else {
                        //cout << " : is NOT a double" << endl;
                    }

                }
            }
            while (iss);
        }
        //cout << "-----------------------" << endl;
    }

    nSteps_ = data_.size()-1;
    gridSize_ = data_[data_.size()-1].size();

}


void Profile :: checkData(void)
{
    vector <float> min;
    vector <float> max;

    int nComps = data_[0][0].size(); // components (coord) of the first line of data;
    int gridSize = data_[0].size(); // grid size of the first step 

    for (auto i = 0; i < nComps; i++ ) {
        min.push_back(data_[0][0][i]);
        max.push_back(data_[0][0][i]);
    }


    if ( coordinates_.size() == 0 )
        for (auto i = 0; i < nComps; i++ ) {
            string coord = string() + char(47+i);
            coordinates_.push_back(coord);
        }


    for (auto i = 0; i < data_.size()-1; i++) {
        if ( gridSize != data_[i].size() ) {
            cout << "File format error: Wrong grid size!" << endl;
            cout << "                 : at step: " << i << endl;
        }

        for (auto j = 0; j < data_[i].size(); j++) {
            if ( nComps != data_[i][j].size() ) {
                cout << "File format error: Wrong component number!" << endl;
                cout << "                 : at step: " << i << endl;
                cout << "                 : at point: " << j << endl;
            }

            for (auto k = 0; k < data_[i][j].size(); k++) {
                if ( data_[i][j][k] < min[k] ) min[k] = data_[i][j][k];
                if ( data_[i][j][k] > max[k] ) max[k] = data_[i][j][k];
            }
        }
    }

    for (auto i = 0; i < nComps; i++ ) {
        inputComps_.push_back(new Component(i, coordinates_[i], min[i], max[i], colors_[i]));
    }

    gridSize_ = gridSize;

	if ( debug_ ) showData();
}


void Profile :: verboseData(void)
{
	if ( verbose_ ) 
	{
		cout << "Title: " << title_ << endl;
		cout << "Componentes:  " << nComps_ << endl;
		cout << "Plot steps: " << nSteps_ << endl;
		cout << "Grid size: " << gridSize_ << endl << endl;

		for (auto i = 0; i < nComps_; i++)
		{
			cout << "Component: " << i+1 << endl;
			cout << "   Name: " << inputComps_[i]->name() << endl;
			cout << "   Min: " << inputComps_[i]->min() << endl;
			cout << "   Max: " << inputComps_[i]->max() << endl;
			cout << "   Color: " << inputComps_[i]->color() << endl;
		}
		cout << endl;
	}
}


void Profile :: showData(void)
{

    cout << "==============================" << endl;
    cout << "Title = " << title_ << endl;

    for (auto i = 0; i < coordinates_.size(); i++)
        cout << "Coordinate[" << i << "] = " << coordinates_[i] << endl;

    for (auto i = 0; i < colors_.size(); i++)
        cout << "Colors[" << i << "] = " << colors_[i] << endl;

    for (auto i = 0; i < stepTime_.size(); i++)
        cout << "Time[" << i << "] = " << stepTime_[i] << endl;

    cout << "==============================" << endl;

    cout.precision(6);
    cout << scientific;
    for (auto i = 0; i < data_.size(); i++) {
		for (auto j = 0; j < data_[i].size(); j++) {
			for (auto k = 0; k < data_[i][j].size(); k++) {
				cout << data_[i][j][k] << " ";
			}
			cout << endl;
		}
		cout << endl;
    }
}



//-------------------------------------------


int Profile :: readFile()
{
	//vector <string> fileStr;
	//string str;

	//extractComponents();

        cout << "Reading input file... " << endl; 
        extractData();
        cout << "Done." << endl; 
        cout << "Checking input file... " << endl; 
        checkData();        
        cout << "Done." << endl; 
        verboseData();

/* 
	THIS CODE WILL BE USED TO READ THE ROLE CONTENT OF THE INPUT FILE SO
	GNUPLOT WILL RECEIVE THE CURVE COORDS NOT THE INPUT FILE ITSELF.

	ifstream In(fileName_.c_str());

	while ( ! In.eof() )
	{
		getline (In, str);
		fileStr.push_back(str);
	}
*/

	return 0;
}


string Profile :: fileName()
{
	return fileName_;
}


string Profile :: title()
{
	return title_;
}


string Profile :: argvComp(int i)
{
	return argvComps_[i];
}


argvOpt Profile :: argvType(int i)
{
	return argvTypes_[i];
}


int Profile :: nProjs()
{
	return argvComps_.size();
}

vector <vector <vector <float> > > * Profile :: data(void)
{
	return & data_;
}

vector <Component *> & Profile :: inputComps(void)
{
	return inputComps_;
}


Component * Profile :: inputComp(int i)
{
	return inputComps_[i];
}


int Profile :: nComps()
{
	return nComps_;
}


int Profile :: nSteps()
{
	return nSteps_;
}


int Profile :: gridSize()
{
	return gridSize_;
}


int Profile :: index()
{
	return index_;
}


int Profile :: ticMarks()
{
	return ticMarks_;
}


int Profile :: legendPos()
{
	return legendPos_;
}


string Profile :: legend()
{
	switch (legendPos_) {
	case 0:
		return "right";
		break;
	case 1:
		return "left";
		break;
	case 2:
		return "off";
		break;
	}
}


bool Profile :: decoration()
{
	return decoration_;
}


bool Profile :: grid()
{
	return grid_;
}


bool Profile :: verbose()
{
	return verbose_;
}


bool Profile :: debug()
{
	return debug_;
}


float Profile :: border()
{
	return border_;
}


float Profile :: lineWidth()
{
	return lineWidth_;
}


float Profile :: printXdim(void)
{
	return printXdim_;
}


float Profile :: printYdim(void)
{
	return printYdim_;
}


int Profile :: fontSize(void)
{
	return fontSize_;
}


void Profile :: index(int i_)
{
	index_ = i_;
}


void Profile :: decoration(bool d_)
{
	decoration_ = d_;
}


void Profile :: grid(bool g_)
{
	grid_ = g_;
}


void Profile :: border(float b_)
{
	border_ = b_;
}


void Profile :: lineWidth(float b_)
{
	lineWidth_ = b_;
}


void Profile :: printXdim(float d_)
{
	printXdim_ = d_;
}


void Profile :: printYdim(float d_)
{
	printYdim_ = d_;
}


void Profile :: fontSize(int f_)
{
	fontSize_ = f_;
}


void Profile :: toggleTicMarks()
{
	ticMarks_ += 1;
	if ( ticMarks_ > 2 )
		ticMarks_ = 0;
}


void Profile :: toggleLegend()
{
	legendPos_ += 1;
	if ( legendPos_ > 2 )
		legendPos_ = 0;
}


void Profile :: toggleDecoration()
{
	if ( decoration_ )
		decoration_ = false;
	else
		decoration_ = true;
}


void Profile :: toggleGrid()
{
	if ( grid() ) 
	{
		cout << "- GRID ON()" << endl;
	}
	else
	{
		cout << "- GRID OFF()" << endl;
	}


	if ( grid_ )
		grid_ = false;
	else
		grid_ = true;
}


void Profile :: toggleVerbose()
{
	if ( verbose_ )
		verbose_ = false;
	else
		verbose_ = true;
}


void Profile :: printUsage(int exitCode = -1)
{
	cerr << "\n    Usage: nplot -i <data-file> [ -c <color-list>] [-p <component-list>] [-t <component-list>] [...] \n\n";

	if ( exitCode != -1 )
		exit(exitCode);

}


string Profile :: frame(int i)
{
	return frame_[i];
}


