#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>

using namespace std;


string title;
vector <string> coordinates;
vector <string> colors;
vector <float> stepTime;
vector <vector <vector <float> > > data; // first dim = step; second dim = point; third dim = coordinate ( or component )


double stringToDouble( const std::string& s )
{
    std::istringstream i(s);
    double x;
    if (!(i >> x))
        return 0;
    return x;
} 


bool checkDouble( const std::string& s )
{
    std::istringstream i(s);
    double x;
    if (!(i >> x))
        return false;
    return true;
} 


bool stripeInfo(string &line, string tag)
{

    auto titlePos = line.find(tag);
    auto titleFound = false;

    if (titlePos != std::string::npos) {
        //cout << "TAG " << tag << " FOUND!!!!" << endl;
        titleFound = false;
        auto const pos = line.find_last_of(':');
        line = line.substr(pos+1);

        return true;
    }

    return false;

}


std::string ReplaceAll(std::string str, const std::string& from, const std::string& to)
{
    size_t start_pos = 0;

    while((start_pos = str.find(from, start_pos)) != std::string::npos) {
        str.replace(start_pos, from.length(), to);
        start_pos += to.length(); // Handles case where 'to' is a substring of 'from'
    }

    return str;
}


bool isEmpty(std::string str)
{

    if ( ReplaceAll(str, " ", "").empty() )
        return true;
    else
        return false;

} 


void extractInfoFromComment(string line)
{
    //cout << "check comment info in: " << line << endl;

    auto titleFound = stripeInfo(line, "Title");
    auto coordinatesFound = stripeInfo(line, "Coordinates");
    auto colorsFound = stripeInfo(line, "Colors");
    auto stepTimeFound = stripeInfo(line, "Time");

    istringstream iss(line);

    if ( titleFound || coordinatesFound || colorsFound || stepTimeFound )
    {
        do
        {
            string word;
            iss >> word;
            auto i = 0;

            if ( !word.empty() ) 
            {
                if ( titleFound )
                {
                    //cout << "Title: " << word << endl;
                    title += word;
                }

                if ( coordinatesFound )
                {
                    //cout << "Coordinate[" << coordinates.size() << "]: " << word << endl;
                    coordinates.push_back(word);
                }

                if ( colorsFound )
                {
                    //cout << "Colors[" << colors.size() << "]: " << word << endl;
                    colors.push_back(word);
                }

                if ( stepTimeFound )
                {
                    //cout << "Time: " << word << endl;
                    stepTime.push_back(stringToDouble(word));
                }
            }
        }
        while (iss);
    }
}


void extractData(void)
{

    ifstream file;
    file.open ("testc.dat");

    string line; line.clear();    
    auto commentFound = false;
    auto emptyLineFound = false;

    //cout << "-----------------------" << endl;

    // Read file line by line
    while (!file.eof())
    {
        getline(file, line);

        // Check if it is a comment line
        auto commentPos = line.find("#");
        if ( commentPos == 0 ) {
            //cout << "This is a comment. " << endl;
            commentFound = true;

            // Check if it has any information tag
            extractInfoFromComment(line);

        }
        else {
            //cout << "This is NOT a comment. " << endl;
            commentFound = false;
        }


        // Check if the line is empty (will start new group of data,e.i.a new "step") 
        if ( isEmpty(line) ) {
            if ( !emptyLineFound ) 
            {
                vector<vector <float> > step;
                data.push_back(step);  // Create a "step" in data multi dim vector
                emptyLineFound = true;
            }
            //cout << "EMPTY LINE" << endl;
        } 
        // If line is not empty and not a comment so it contains data (coordinates)
        else if ( !commentFound ) {
            istringstream iss(line);
            emptyLineFound = false;

            vector<float> row;
            data[data.size()-1].push_back(row);

            // Split line into words
            do
            {
                string word;
                iss >> word;

                if ( !word.empty() )
                {
                    //cout << "Word: " << word;
                    if ( checkDouble(word) ) {
                        //cout << " : is a double" << endl;
                        data[data.size()-1][ data[data.size() -1].size()-1 ].push_back(stringToDouble(word));
                    }
                    else {
                        //cout << " : is NOT a double" << endl;
                    }

                }
            }
            while (iss);
        }
        //cout << "-----------------------" << endl;
    }

}


void show(void)
{

    cout << "==============================" << endl;
    cout << "Title = " << title << endl;

    for (auto i = 0; i < coordinates.size(); i++)
        cout << "Coordinate[" << i << "] = " << coordinates[i] << endl;

    for (auto i = 0; i < colors.size(); i++)
        cout << "Colors[" << i << "] = " << colors[i] << endl;

    for (auto i = 0; i < stepTime.size(); i++)
        cout << "Time[" << i << "] = " << stepTime[i] << endl;

    cout << "==============================" << endl;


    cout.precision(6);
    cout << scientific;
    for (auto i = 0; i < data.size(); i++) {
    for (auto j = 0; j < data[i].size(); j++) {
    for (auto k = 0; k < data[i][j].size(); k++) {
        cout << data[i][j][k] << " ";
    }
    cout << endl;
    }
    cout << endl;
    }

}


/*
int main (void) 
{
    extractData();

    show();
}

*/

#include <iostream>
#include <sstream>
#include <algorithm>
#include <vector>
#include <string>

using namespace std;

struct coords
{
    int index;
    string name;
};

bool anotherWayToSort(coords & i, coords & j) { return i.name.size() > j.name.size(); }

int main()
{

    string eq2("x * xot + yv - 2 * y * xy + xy");
    vector<string> stringVec2 = {"x", "xot", "y", "xy", "xya", "yv", "yxt", "xyt"};
    vector <coords> cVec(stringVec2.size());

    for (auto i=0; i<stringVec2.size(); i++) {
        cVec[i].index = i;
        cVec[i].name = stringVec2[i];
    }
    
    // Sorting the string vector
    sort(cVec.begin(), cVec.end(), anotherWayToSort);

    cout << eq2 << endl;
    for (auto i=0; i<cVec.size(); i++) {
        stringstream ss;
        ss << cVec[i].index;
        string str = ss.str();

        eq2 = ReplaceAll(eq2, cVec[i].name, "$"+str);
    }
    cout << eq2 << endl;

    return 0;
}



