#include <string>
#include <PlotInlineCommand.h>

using namespace std;


PlotInlineCommand :: PlotInlineCommand() :
	name("Plot")
{
}


string PlotInlineCommand :: execute(Projection *projection, Profile *profile)
{
	return execute (projection, profile, profile->index());
}


string PlotInlineCommand :: execute(Projection *projection, Profile *profile, int index_)
{
	string command, withTitle, compName, index;
	Gnuplot * gplt = projection->projection();
	stringstream lineStyle, ii;


	if ( profile->decoration() )
		withTitle = "";
	else
		withTitle = "no";

	ii << index_;
	index = ii.str();

	compName = projection->compToLabel(1);
	command  = "plot '-' ";
	//command += " using ($1):($2)";
	command += " using (" + projection->compToProj(0) + "):(" +  projection->compToProj(1) + ")";
	command += " " + withTitle + "title '" + compName + "' w l ls 1";

	for (int j = 2; j < projection->nComps(); j++)
	{
		lineStyle << j;
		compName = projection->compToLabel(j);
		command += ",    '-' ";
		//command += " using ($1):($2)";
		command += " using (" + projection->compToProj(0) + "):(" +  projection->compToProj(j) + ")";
		command += " " + withTitle + "title '" + compName + "' w l ls " + lineStyle.str();
		lineStyle.str(string());
	}

	if ( profile->verbose() ) cout << "Command: " << command << endl;

	command += "\n";
    for (auto j = 0; j < projection->data_->at(index_).size(); j++) {
		stringstream xx, yy;
		xx << projection->data_->at(index_).at(j).at(projection->compToIndex(0));
		yy << projection->data_->at(index_).at(j).at(projection->compToIndex(1));
		command += xx.str() + " " + yy.str() + "\n";
	}
	command += "e\n";


	for (int k = 2; k < projection->nComps(); k++)
	{
		for (auto j = 0; j < projection->data_->at(index_).size(); j++) {
			stringstream xx, yy;
			xx << projection->data_->at(index_).at(j).at(projection->compToIndex(0));
			yy << projection->data_->at(index_).at(j).at(projection->compToIndex(k));
			command += xx.str() + " " + yy.str() + "\n";
		}
		command += "e\n";
	}

	// add particular projection commands
	for (int j=0; j <projection->nCmd(); j++)
		command += projection->cmd(j)->execute(projection, profile);

	// DEBUG
	if ( profile->debug() ) cout << "Command: " << command << endl;
	//if ( profile->verbose() ) cout << "Command: " << command << endl;

	if ( profile->debug()) {
		cout.precision(6);
		cout << scientific;
		cout << "Data:  " << projection->data_->size() << endl;
		for (auto i = 0; i < projection->data_->size(); i++) {
			for (auto j = 0; j < projection->data_->at(i).size(); j++) {
				for (auto k = 0; k < projection->data_->at(i).at(j).size(); k++) {
					cout << "        " << k << " - k" ;
					cout << "        " << projection->data_->at(i).at(j).at(k) << " ";
					cout << endl;
				}
				cout << "    " << j << " - j" ;
				cout << endl;
			}
			cout << i << " - i" ;
			cout << endl;
		}
	}

	//gplt->cmd (command.c_str());
	return command;

}


	/* Example of a gnuplot command:
		plot 'data.dat' using 1:2      title 'data 1', \
     		''         u     1:3      t     'data 2', \
     		''         u     1:4      t     'data 3', \
     		''         u 1:($2+$3+$4) t     'sum of datas'
	*/


	/* Example of a gnuplot command with inline data:
		plot '-' u 1:2 w l title "data1", '-' u 1:2 w l title "data2"
			1 2
			2 5
			3 7
			e
			1 3
			2 4
			3 9
			e
	*/
